import _objectWithoutProperties from "@babel/runtime/helpers/objectWithoutProperties";
import _slicedToArray from "@babel/runtime/helpers/slicedToArray";
import _regeneratorRuntime from "@babel/runtime/regenerator";
import _asyncToGenerator from "@babel/runtime/helpers/asyncToGenerator";
import React from 'react';
import { makeCancelablePromise } from '@billogram/cancelable-promise';
import dataFetched from './dataFetched';
import dataFetchedError from './dataFetchedError';
import dataUpdate from './dataUpdate';

function noop() {}

export default function createDataProvider() {
  var FoundationDataProviderContext = /*#__PURE__*/React.createContext({
    fetch: function () {
      var _fetch = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee() {
        return _regeneratorRuntime.wrap(function _callee$(_context) {
          while (1) {
            switch (_context.prev = _context.next) {
              case 0:
              case "end":
                return _context.stop();
            }
          }
        }, _callee);
      }));

      function fetch() {
        return _fetch.apply(this, arguments);
      }

      return fetch;
    }(),
    result: {
      state: 'fetching-initial'
    },
    update: noop
  });

  function FoundationDataProvider(_ref) {
    var children = _ref.children,
        fetchData = _ref.fetchData,
        initialFetchDataOptions = _ref.initialFetchDataOptions;
    var cancel = React.useRef(noop);

    var _React$useState = React.useState({
      state: 'fetching-initial'
    }),
        _React$useState2 = _slicedToArray(_React$useState, 2),
        result = _React$useState2[0],
        setResult = _React$useState2[1];

    var fetch = React.useCallback( /*#__PURE__*/function () {
      var _ref2 = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee2(options) {
        var _makeCancelablePromis, _makeCancelablePromis2, newRequest, newCancel, token, _token, errors;

        return _regeneratorRuntime.wrap(function _callee2$(_context2) {
          while (1) {
            switch (_context2.prev = _context2.next) {
              case 0:
                _context2.prev = 0;
                cancel.current();
                setResult(function (result) {
                  switch (result.state) {
                    case 'error-initial':
                    case 'fetching-initial':
                      {
                        return {
                          state: 'fetching-initial'
                        };
                      }

                    default:
                      {
                        return {
                          data: result.data,
                          state: 'fetching'
                        };
                      }
                  }
                });
                _makeCancelablePromis = makeCancelablePromise(fetchData(options)), _makeCancelablePromis2 = _slicedToArray(_makeCancelablePromis, 2), newRequest = _makeCancelablePromis2[0], newCancel = _makeCancelablePromis2[1];
                cancel.current = newCancel;
                _context2.next = 7;
                return newRequest;

              case 7:
                token = _context2.sent;

                if (!token.isPromiseCanceled) {
                  setResult(dataFetched(token.value));
                }

                _context2.next = 15;
                break;

              case 11:
                _context2.prev = 11;
                _context2.t0 = _context2["catch"](0);
                _token = _context2.t0;

                if (!_token.isPromiseCanceled) {
                  // eslint-disable-next-line @typescript-eslint/no-unnecessary-condition
                  errors = _token.reason !== void 0 && _token.reason.data !== void 0 ? _token.reason.data : {};
                  setResult(dataFetchedError(errors));
                }

              case 15:
              case "end":
                return _context2.stop();
            }
          }
        }, _callee2, null, [[0, 11]]);
      }));

      return function (_x) {
        return _ref2.apply(this, arguments);
      };
    }(), [fetchData]);
    var update = React.useCallback(function (data) {
      setResult(dataUpdate(data));
    }, []);
    React.useEffect(function () {
      fetch(initialFetchDataOptions);
      return function () {
        cancel.current();
        cancel.current = noop;
      };
    }, [fetch, initialFetchDataOptions]);
    var context = React.useMemo(function () {
      return {
        fetch: fetch,
        result: result,
        update: update
      };
    }, [fetch, result, update]);
    return /*#__PURE__*/React.createElement(FoundationDataProviderContext.Provider, {
      value: context
    }, children);
  }

  function FoundationDataConsumer(_ref3) {
    var children = _ref3.children;
    return /*#__PURE__*/React.createElement(FoundationDataProviderContext.Consumer, null, function (context) {
      return children(context);
    });
  }

  function FoundationDataProsumer(props) {
    var children = props.children,
        providerProps = _objectWithoutProperties(props, ["children"]);

    return /*#__PURE__*/React.createElement(FoundationDataProvider, providerProps, /*#__PURE__*/React.createElement(FoundationDataConsumer, null, children));
  }

  function useDataProvider() {
    return React.useContext(FoundationDataProviderContext);
  }

  return {
    FoundationDataConsumer: FoundationDataConsumer,
    FoundationDataProsumer: FoundationDataProsumer,
    FoundationDataProvider: FoundationDataProvider,
    useDataProvider: useDataProvider
  };
}
//# sourceMappingURL=createDataProvider.js.map